Package: iterators
Type: Package
Title: Provides Iterator Construct
Version: 1.0.14
Authors@R: c(person("Folashade", "Daniel", role="cre", email="fdaniel@microsoft.com"),
             person("Revolution", "Analytics", role=c("aut", "cph")),
             person("Steve", "Weston", role="aut"))
Description: Support for iterators, which allow a programmer to traverse
    through all the elements of a vector, list, or other collection of data.
URL: https://github.com/RevolutionAnalytics/iterators
Depends: R (>= 2.5.0), utils
Suggests: RUnit, foreach
License: Apache License (== 2.0)
NeedsCompilation: no
Packaged: 2022-01-16 18:19:31 UTC; folashade
Author: Folashade Daniel [cre],
  Revolution Analytics [aut, cph],
  Steve Weston [aut]
Maintainer: Folashade Daniel <fdaniel@microsoft.com>
Repository: CRAN
Date/Publication: 2022-02-05 00:50:08 UTC
Built: R 4.4.0; ; 2026-05-12 01:24:38 UTC; unix

Package: iterators
Type: Package
Title: Provides Iterator Construct
Version: 1.0.14
Authors@R: c(person("Folashade", "Daniel", role="cre", email="fdaniel@microsoft.com"),
             person("Revolution", "Analytics", role=c("aut", "cph")),
             person("Steve", "Weston", role="aut"))
Description: Support for iterators, which allow a programmer to traverse
    through all the elements of a vector, list, or other collection of data.
URL: https://github.com/RevolutionAnalytics/iterators
Depends: R (>= 2.5.0), utils
Suggests: RUnit, foreach
License: Apache License (== 2.0)
NeedsCompilation: no
Packaged: 2022-01-16 18:19:31 UTC; folashade
Author: Folashade Daniel [cre],
  Revolution Analytics [aut, cph],
  Steve Weston [aut]
Maintainer: Folashade Daniel <fdaniel@microsoft.com>
Repository: CRAN
Date/Publication: 2022-02-05 00:50:08 UTC

export(iter, nextElem, isplit)
export(irnorm, irunif, irbinom, irnbinom, irpois)
export(icount, idiv, ireadLines, iread.table)
export(makeIwrapper, isample)
export(icountn, iapply)
S3method("iter", "default")
S3method("iter", "iter")
S3method("iter", "matrix")
S3method("iter", "data.frame")
S3method("iter", "function")
S3method("nextElem", "containeriter")
S3method("nextElem", "matrixiter")
S3method("nextElem", "dataframeiter")
S3method("nextElem", "funiter")
S3method("nextElem", "abstractiter")
S3method("as.list", "iter")
S3method("isplit", "default")
S3method("isplit", "data.frame")

# iterators

[![CRAN](https://www.r-pkg.org/badges/version/iterators)](https://cran.r-project.org/package=iterators)
![Downloads](https://cranlogs.r-pkg.org/badges/iterators)

A support package used by [foreach](https://github.com/RevolutionAnalytics/foreach).

library(iterators)

# return an iterator that returns subvectors of a vector.
# can specify either "chunks" or "chunkSize" arguments
# since that is what the "idiv" function supports.
ivector <- function(x, ...) {
  # don't evaluate x if is specified using the ':' operator
  q <- substitute(x)
  if (identical(q[[1]], as.name(':'))) {
    rm(list='x')  # being paranoid: don't want to evaluate promise
    lower <- as.integer(eval.parent(q[[2]]))
    upper <- as.integer(eval.parent(q[[3]]))
    inc <- if (upper >= lower) 1L else -1L
    len <- abs(upper - lower) + 1L
    it <- idiv(len, ...)

    nextEl <- function() {
      n <- nextElem(it)
      y <- seq(lower, by=inc, length=n)
      lower <<- lower + (inc * n)
      y
    }
  } else {
    i <- 1
    it <- idiv(length(x), ...)

    nextEl <- function() {
      n <- nextElem(it)
      ix <- seq(i, length=n)
      i <<- i + n
      x[ix]
    }
  }

  obj <- list(nextElem=nextEl)
  class(obj) <- c('ivector', 'abstractiter', 'iter')
  obj
}

# create a vector iterator that returns three subvectors
it <- ivector(1:25, chunks=3)
print(as.list(it))

# create a vector iterator that returns subvectors
# with a maximum length of 10
it <- ivector(25:1, chunkSize=10)
print(as.list(it))

library(iterators)

ilimit <- function(it, times) {
  it <- iter(it)

  nextEl <- function() {
    if (times > 0)
      times <<- times - 1
    else
      stop('StopIteration')

    nextElem(it)
  }

  obj <- list(nextElem=nextEl)
  class(obj) <- c('ilimit', 'abstractiter', 'iter')
  obj
}

it <- ilimit(icount(Inf), 3)
print(nextElem(it))
print(nextElem(it))
print(nextElem(it))
print(tryCatch(nextElem(it), error=function(e) e))

library(iterators)

# This example was originally written and contributed
# by Hadley Wickham, with minor modifications by
# Revolution Analytics

# Define a hasNext generic function
hasNext <- function(obj, ...) {
  UseMethod('hasNext')
}

# Define a hasNext method for the "ihasNext" class
hasNext.ihasNext <- function(obj, ...) {
  obj$hasNext()
}

# This function takes an iterator and returns an iterator that supports
# the "hasNext" method.  This simplifies manually calling the "nextElem"
# method of the iterator, since you don't have to worry about catching
# the "StopIteration" exception.
ihasNext <- function(it) {
  it <- iter(it)

  # If "it" already has a hasNext function, return it unchanged
  if (!is.null(it$hasNext))
    return(it)

  cache <- NULL
  has_next <- NA

  nextEl <- function() {
    if (!hasNx())
      stop('StopIteration', call.=FALSE)

    # Reset the "has_next" flag and return the value
    has_next <<- NA
    cache
  }

  hasNx <- function() {
    # Check if you already know the answer
    if (!is.na(has_next))
      return(has_next)

    # Try to get the next element
    tryCatch({
      cache <<- nextElem(it)
      has_next <<- TRUE
    },
    error=function(e) {
      if (identical(conditionMessage(e), 'StopIteration')) {
        has_next <<- FALSE
      } else {
        stop(e)
      }
    })

    has_next
  }

  obj <- list(nextElem=nextEl, hasNext=hasNx)
  class(obj) <- c('ihasNext', 'abstractiter', 'iter')
  obj
}

# Create a "counting" iterator that has a hasNext method
it <- ihasNext(icount(3))

# Print the values of the iterator without the need for error handling
while (hasNext(it))
  print(nextElem(it))

library(iterators)

# return an iterator that returns subvectors of a vector.
# can specify either "chunks" or "chunkSize" arguments
# since that is what the "idiv" function supports.
ivector <- function(x, ...) {
 i <- 1
 it <- idiv(length(x), ...)

 nextEl <- function() {
   n <- nextElem(it)
   ix <- seq(i, length=n)
   i <<- i + n
   x[ix]
 }

 obj <- list(nextElem=nextEl)
 class(obj) <- c('ivector', 'abstractiter', 'iter')
 obj
}

# create a vector iterator that returns three subvectors
it <- ivector(1:25, chunks=3)
print(as.list(it))

# create a vector iterator that returns subvectors
# with a maximum length of 10
it <- ivector(1:25, chunkSize=10)
print(as.list(it))

library(iterators)

# return an iterator that returns the specified value
# a limited number of times
irep <- function(x, times) {
  nextEl <- function() {
    if (times > 0)
      times <<- times - 1
    else
      stop('StopIteration')

    x
  }

  obj <- list(nextElem=nextEl)
  class(obj) <- c('irep', 'abstractiter', 'iter')
  obj
}

# create an iterator that returns a 7 exactly 6 times
it <- irep(7, 6)

# convert the iterator into a list, which gets all of its values
print(unlist(as.list(it)))

library(iterators)

# return an iterator that returns subvectors of a sequence
# of a specified length.
# can specify either "chunks" or "chunkSize" arguments
# since that is what the "idiv" function supports.
iseq <- function(n, ...) {
 i <- 1
 it <- idiv(n, ...)

 nextEl <- function() {
   n <- nextElem(it)
   x <- seq(i, length=n)
   i <<- i + n
   x
 }

 obj <- list(nextElem=nextEl)
 class(obj) <- c('iseq', 'abstractiter', 'iter')
 obj
}

# create a sequence iterator that returns three subvectors
it <- iseq(25, chunks=3)
print(as.list(it))

# create a sequence iterator that returns subvectors
# with a maximum length of 10
it <- iseq(25, chunkSize=10)
print(as.list(it))

library(iterators)

# return an iterator that returns the specified value forever
iforever <- function(x) {
  nextEl <- function() x
  obj <- list(nextElem=nextEl)
  class(obj) <- c('iforever', 'abstractiter', 'iter')
  obj
}

# create an iterator that returns 42 forever
it <- iforever(42)

# call it three times
for (i in 1:3)
  print(nextElem(it))

library(iterators)

# Returns an iterator that limits another iterator based on time
itimer <- function(it, time) {
  it <- iter(it)
  start <- proc.time()[[3]]

  nextEl <- function() {
    current <- proc.time()[[3]]
    if (current - start >= time)
      stop('StopIteration')

    nextElem(it)
  }

  obj <- list(nextElem=nextEl)
  class(obj) <- c('itimer', 'abstractiter', 'iter')
  obj
}

# Create a iterator that counts for one second
it <- itimer(icount(Inf), 1)
tryCatch({
  repeat {
    print(nextElem(it))
  }
},
error=function(e) {
  cat('timer expired\n')
})

library(iterators)

# Returns a filtering iterator
ifilter <- function(it, FUN, ...) {
  it <- iter(it)

  nextEl <- function() {
    repeat {
      x <- nextElem(it)
      if (FUN(x, ...))
        break
    }
    x
  }

  obj <- list(nextElem=nextEl)
  class(obj) <- c('ifilter', 'abstractiter', 'iter')
  obj
}

# Simple example use
it <- irnorm(1, count=10)
is.positive <- function(x) x > 0
print(as.list(ifilter(it, is.positive)))

# Example using a function with an additional argument
it <- irnorm(1, count=10)
greater.than <- function(x, y) x > y
print(as.list(ifilter(it, greater.than, 1.0)))

## unit tests will not be done if RUnit is not available
if(require("RUnit", quietly=TRUE)) {
 
  ## --- Setup ---
 
  pkg <- "iterators" # <-- Change to package name!
  
  if(Sys.getenv("RCMDCHECK") == "FALSE") {
    ## Path to unit tests for standalone running under Makefile (not R CMD check)
    ## PKG/tests/../inst/unitTests
    path <- file.path(getwd(), "..", "inst", "unitTests")
  } else {
    ## Path to unit tests for R CMD check
    ## PKG.Rcheck/tests/../PKG/unitTests
    path <- system.file(package=pkg, "unitTests")
  }
  cat("\nRunning unit tests\n")
  print(list(pkg=pkg, getwd=getwd(), pathToUnitTests=path))
 
  library(package=pkg, character.only=TRUE)
  ################################################################
  ## BEGIN PACKAGE SPECIFIC CONFIGURATION                        #
  ################################################################
  
 
  ################################################################
  ## END PACKAGE SPECIFIC CONFIGURATION                          #
  ################################################################
 
  ## If desired, load the name space to allow testing of private functions
  ## if (is.element(pkg, loadedNamespaces()))
  ##     attach(loadNamespace(pkg), name=paste("namespace", pkg, sep=":"), pos=3)
  ##
  ## or simply call PKG:::myPrivateFunction() in tests

 
 
  ## --- Testing ---
 
  ## Define tests
  testSuite <- defineTestSuite(name=paste(pkg, "unit testing"),
                                          dirs=path, testFileRegexp = "^.+Test\\.R$")
  ## Run
  tests <- runTestSuite(testSuite)
 
  ## Default report name
  pathReport <- file.path(tempdir(), "report")
 
  ## Report to stdout and text files
  cat("------------------- UNIT TEST SUMMARY ---------------------\n\n")
  printTextProtocol(tests, showDetails=FALSE)
  printTextProtocol(tests, showDetails=FALSE,
                    fileName=paste(pathReport, "Summary.txt", sep=""))
  printTextProtocol(tests, showDetails=TRUE,
                    fileName=paste(pathReport, ".txt", sep=""))
 
  ## Report to HTML file
  printHTMLProtocol(tests, fileName=paste(pathReport, ".html", sep=""))
#  printHTMLProtocol(tests, fileName=file.path(dirname(dirname(getwd())),pkg,"gsDesign-RUnit-Test-Summary.html")) #paste(pathReport, ".html", sep=""))  
 
  ## Return stop() to cause R CMD check stop in case of
  ##  - failures i.e. FALSE to unit tests or
  ##  - errors i.e. R errors
  tmp <- getErrors(tests)
  if(tmp$nFail > 0 | tmp$nErr > 0) {
    stop(paste("\n\nunit testing failed (#test failures: ", tmp$nFail,
               ", #R errors: ",  tmp$nErr, ")\n\n", sep=""))
  }
} else {
  warning("cannot run unit tests -- package RUnit is not available")
}

#
# Copyright (c) 2008-2010 Revolution Analytics
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# This function makes iterator makers.  The resulting iterator makers all take
# an optional "count" argument which specifies the number of times the
# resulting iterator should fire.  The iterators are wrappers around functions
# that return different values each time they are called.  All this is done to
# avoid cutting and pasting the same code repeatedly.  
makeIwrapper <- function(FUN) {
  function(..., count) {
    if (!missing(count) && (!is.numeric(count) || length(count) != 1))
      stop('count must be a numeric value')

    # construct the call object to put into the nextElem function
    m <- as.call(c(as.name(FUN), list(...)))

    # construct the body of the nextElem function
    fbody <- if (missing(count)) {
      m
    } else {
      substitute({
        if (count > 0) {
          count <<- count - 1L
          REPLACETHIS
        } else {
          stop('StopIteration', call.=FALSE)
        }
      }, list(REPLACETHIS=m))
    }

    # create the nextElem function using fbody
    nextEl <- function() NULL
    body(nextEl) <- fbody

    # create and return the iterator object
    it <- list(nextElem=nextEl)
    class(it) <- c('abstractiter', 'iter')
    it
  }
}

# define some iterator makers using makeIwrapper
irunif <- makeIwrapper('runif')
irnorm <- makeIwrapper('rnorm')
irbinom <- makeIwrapper('rbinom')
irnbinom <- makeIwrapper('rnbinom')
irpois <- makeIwrapper('rpois')
isample <- makeIwrapper('sample')  

# a counting iterator
icount <- function(count) {
  if (missing(count))
    count <- NULL
  else if (!is.numeric(count) || length(count) != 1)
    stop('count must be a numeric value')

  i <- 0L

  nextEl <- function() {
    if (is.null(count) || i < count)
      (i <<- i + 1L)
    else
      stop('StopIteration', call.=FALSE)
  }

  it <- list(nextElem=nextEl)
  class(it) <- c('abstractiter', 'iter')
  it
}

# an iterator over pieces of a number
idiv <- function(n, ..., chunks, chunkSize) {
  if (!is.numeric(n) || length(n) != 1)
    stop('n must be a numeric value')

  if (length(list(...)) > 0)
    stop('chunks and chunkSize must be specified as named arguments')

  if ((missing(chunkSize) && missing(chunks)) ||
      (!missing(chunkSize) && !missing(chunks)))
    stop('either chunks or chunkSize must be specified, but not both')

  if (missing(chunks)) {
    if (!is.numeric(chunkSize) || length(chunkSize) != 1 || chunkSize < 1)
      stop('chunkSize must be a numeric value >= 1')
    chunks <- ceiling(n / chunkSize)
  }

  nextEl <- function() {
    if (chunks <= 0 || n <= 0)
      stop('StopIteration', call.=FALSE)

    m <- ceiling(n / chunks)
    n <<- n - m
    chunks <<- chunks - 1
    m
  }

  it <- list(nextElem=nextEl)
  class(it) <- c('abstractiter', 'iter')
  it
}

# an iterator over text lines from a connection
ireadLines <- function(con, n=1, ...) {
  if (!is.numeric(n) || length(n) != 1 || n < 1)
    stop('n must be a numeric value >= 1')

  if (is.character(con)) {
    con <- file(con, open='r')
    doClose <- TRUE
  } else {
    doClose <- FALSE
  }

  nextEl <- function() {
    if (is.null(con))
      stop('StopIteration', call.=FALSE)

    r <- readLines(con, n=n, ...)
    if (length(r) == 0) {
      if (doClose)
        close(con)
      con <<- NULL
      stop('StopIteration', call.=FALSE)
    }
    r
  }

  it <- list(nextElem=nextEl)
  class(it) <- c('abstractiter', 'iter')
  it
}

# an iterator over rows of a data frame read from a file
iread.table <- function(file, ..., verbose=FALSE) {
  args <- list(...)
  argnames <- names(args)

  # need to do this (at least for now) because the default values for
  # header and row.names depend on the first few lines of the file,
  # which could cause a different number of columns to be returned from
  # the first versus the subsequent calls to read.table
  if (!all(c('header', 'row.names') %in% argnames))
    stop('both header and row.names must be specified in this implementation')

  nrows <- if ('nrows' %in% argnames) args$nrows else 1
  row.names <- args$row.names

  # it doesn't seem to make sense to allow nrows < 1 for the "iterator"
  # version of read.table
  if (!is.numeric(nrows) || length(nrows) != 1 || nrows < 1)
    stop('nrows must be a numeric value >= 1')

  # open the file if necessary and remember to close it
  if (is.character(file)) {
    file <- file(file, open='r')
    doClose <- TRUE
  } else {
    doClose <- FALSE
  }

  # create the call object that we'll use to call read.table
  m <- as.call(c(as.name('read.table'), file='', list(...)))
  m$file <- file
  m$nrows <- nrows  # needed since we use a different default than read.table
  env <- sys.frame(sys.nframe())

  # compute these once rather than repeatedly
  rnlen <- length(row.names)
  gotrownames <- is.character(row.names) && rnlen > 1

  # initialize a few state variables
  first.time <- TRUE
  irow <- 1
  errmsg <- NULL

  nextEl <- function() {
    if (!is.null(errmsg))
      stop(paste('iterator failed previously:', errmsg), call.=FALSE)

    if (is.null(file))
      stop('StopIteration', call.=FALSE)

    if (gotrownames) {
      rem <- rnlen - irow + 1  # remaining strings in row.names
      nrows <<- min(nrows, rem)  # possibly decrease nrows to match row.names

      # there is a problem if nrows is one: we would have to set row.names
      # to a character vector of length one, which is interpreted
      # incorrectly by read.table
      if (nrows > 1)
        m$row.names <<- row.names[seq(irow, length=nrows)]
      else
        m['row.names'] <<- list(NULL)  # we'll fix the row names later
      m$nrows <<- nrows
    }

    # call read.table to actually read the file
    r <- tryCatch({
      # handle the case where we've run out of row names
      if (nrows > 0) {
        if (verbose)
          print(m)
        eval(m, env)
      } else {
        NULL
      }
    },
    error=function(e) {
      # this error is thrown at the end of input sometimes
      # but other times a data frame with no rows is returned
      # (for instance when col.names is specified)
      if (!identical(conditionMessage(e), 'no lines available in input')) {
        if (doClose)
          close(file)
        file <<- NULL
        errmsg <<- conditionMessage(e)
        stop(e)
      }
      NULL
    })

    # set header to FALSE, skip to 0, and col.names to names(r)
    # after the first call to read.table
    if (first.time) {
      first.time <<- FALSE
      m$header <<- FALSE
      m$skip <<- 0
      nms <- names(r)
      if (is.numeric(row.names)) {
        nms <- if (row.names == 1)
          c('', nms)
        else if (row.names >= length(nms))
          c(nms, '')
        else
          c(nms[1:(row.names-1)], '', nms[row.names:length(nms)])
      }
      m$col.names <<- nms
    }

    # check if we're done reading
    if (is.null(r) || nrow(r) == 0) {
      if (doClose)
        close(file)
      file <<- NULL
      stop('StopIteration', call.=FALSE)
    }

    if (gotrownames) {
      # fix the row names for this particular case
      if (nrows == 1)
        rownames(r) <- row.names[irow]

      # update the index into row.names
      irow <<- irow + nrows
    }

    r
  }

  it <- list(nextElem=nextEl)
  class(it) <- c('abstractiter', 'iter')
  it
}

# simple test of recycle
test01 <- function() {
  if (require(foreach, quietly=TRUE)) {
    nr <- 21
    nc <- 17
    x <- rnorm(nr)
    it <- iter(x, recycle=TRUE)
    actual <- foreach(y=it, icount(nr*nc), .combine='c') %do% y
    dim(actual) <- c(nr, nc)
    expected <- matrix(x, nr, nc)
    checkEquals(actual, expected)
  }
}

#
# Copyright (c) 2008-2010 Revolution Analytics
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

icountn <- function(vn) {
  n <- length(vn)
  if (n == 0)
    stop('illegal zero length vector')

  icar <- icount(vn[n])
  if (n > 1) {
    icdr <- icountn(vn[-n])
    hasVal <- FALSE
    nextVal <- NULL
  }

  nextEl <- if (n == 1) {
    function() nextElem(icar)
  } else {
    function() {
      repeat {
        if (!hasVal) {
          nextVal <<- nextElem(icar)
          hasVal <<- TRUE
        }

        tryCatch({
          return(c(nextElem(icdr), nextVal))
        },
        error=function(e) {
          if (identical(conditionMessage(e), 'StopIteration')) {
            icdr <<- icountn(vn[-n])
            hasVal <<- FALSE
          } else {
            stop(e)
          }
        })
      }
    }
  }

  structure(list(nextElem=nextEl), class=c('abstractiter', 'iter'))
}

iwhich <- function(nf, ind) {
  n <- length(ind)
  if (n == 0)
    stop('illegal zero length vector')

  x <- rep(TRUE, length(nf[[1]]))
  for (i in seq_len(n))
    x <- x & nf[[i]] == ind[i]

  which(x)
}

# define the generic function
isplit <- function(x, f, drop=FALSE, ...) {
  UseMethod('isplit')
}

# define the default method
isplit.default <- function(x, f, drop=FALSE, ...) {
  if (!is.list(f)) f <- list(f)
  cf <- lapply(f, function(a) if (is.factor(a)) a else as.factor(a))
  nf <- lapply(cf, as.integer)
  flevels <- lapply(f, function(a) if (is.factor(a)) levels(a) else sort(unique.default(a)))
  it <- icountn(unlist(lapply(cf, nlevels)))

  nextEl <- function() {
    repeat {
      i <- nextElem(it)
      j <- iwhich(nf, i)
      if (!drop || length(j) > 0)
        break
    }
    k <- seq_along(i)
    names(k) <- names(cf)
    key <- lapply(k, function(x) flevels[[x]][i[x]])
    list(value=x[j], key=key)
  }

  structure(list(nextElem=nextEl), class=c('abstractiter', 'iter'))
}

# define the data frame method which uses the default method
isplit.data.frame <- function(x, f, drop=FALSE, ...) {
  it <- isplit(seq_len(nrow(x)), f, drop=drop, ...)
  nextEl <- function() {
    i <- nextElem(it)
    list(value=x[i$value,, drop=FALSE], key=i$key)
  }
  structure(list(nextElem=nextEl), class=c('abstractiter', 'iter'))
}

library(iterators)

# test isplit with a single factor
test01 <- function() {
  x <- rnorm(200)
  f <- factor(sample(1:10, length(x), replace=TRUE))

  it <- isplit(x, f)
  expected <- split(x, f)

  for (i in expected) {
    actual <- nextElem(it)
    checkEquals(actual$value, i)
  }

  it <- isplit(x, f, drop=TRUE)
  expected <- split(x, f, drop=TRUE)

  for (i in expected) {
    actual <- nextElem(it)
    checkEquals(actual$value, i)
  }
}

# test isplit with two factors
test02 <- function() {
  x <- rnorm(200)
  f <- list(factor(sample(1:10, length(x), replace=TRUE)),
            factor(sample(1:10, length(x), replace=TRUE)))

  it <- isplit(x, f)
  expected <- split(x, f)

  for (i in expected) {
    actual <- nextElem(it)
    checkEquals(actual$value, i)
  }

  it <- isplit(x, f, drop=TRUE)
  expected <- split(x, f, drop=TRUE)

  for (i in expected) {
    actual <- nextElem(it)
    checkEquals(actual$value, i)
  }
}

#
# Copyright (c) 2008-2010 Revolution Analytics
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

as.list.iter <- function(x, n=as.integer(2^31-1), ...) {
  size <- 64
  a <- vector('list', length=size)
  i <- 0
  tryCatch({
    while (i < n) {
      if (i >= size) {
        size <- min(2 * size, n)
        length(a) <- size
      }
      a[i + 1] <- list(nextElem(x))
      i <- i + 1
    }
  },
  error=function(e) {
    if (!identical(conditionMessage(e), 'StopIteration'))
      stop(e)
  })

  length(a) <- i
  a
}

#
# Copyright (c) 2008-2010 Revolution Analytics
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

iapply <- function(X, MARGIN) {
  xit <- icountn(dim(X)[MARGIN])

  nextEl <- function() {
    i <- nextElem(xit)
    j <- rep('', length(dim(X)))
    j[MARGIN] <- as.character(i)
    s <- paste('X[', paste(j, collapse=','), ']', sep='')
    x <- parse(text=s)
    eval(x)
  }

  it <- list(nextElem=nextEl)
  class(it) <- c('abstractiter', 'iter')
  it
}

library(iterators)

# test iapply on 3D arrays
test01 <- function() {
  test <- function(actual, it) {
    expected <- nextElem(it)
    checkEquals(expected, actual)
    NULL
  }

  a <- array(1:24, c(2,3,4))

  margins <- list(1, 2, 3,
                  c(1, 2), c(1, 3), c(2, 1), c(2, 3), c(3, 1), c(3, 2),
                  c(1, 2, 3), c(1, 3, 2), c(2, 1, 3), c(2, 3, 1),
                  c(3, 1, 2), c(3, 2, 1))
  for(MARGIN in margins) {
    # cat(sprintf('testing %s\n', paste(MARGIN, collapse=', ')))
    it <- iapply(a, MARGIN)
    apply(a, MARGIN, test, it)
  }
}

# test iapply on matrices
test02 <- function() {
  test <- function(actual, it) {
    expected <- nextElem(it)
    checkEquals(expected, actual)
    NULL
  }

  m <- matrix(1:24, c(6,4))

  margins <- list(1, 2, c(1, 2), c(2, 1))
  for(MARGIN in margins) {
    # cat(sprintf('testing %s\n', paste(MARGIN, collapse=', ')))
    it <- iapply(m, MARGIN)
    apply(m, MARGIN, test, it)
  }
}

#
# Copyright (c) 2008-2010 Revolution Analytics
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# generic function for creating an iterator object
iter <- function(obj, ...) {
  UseMethod('iter')
}

# calling iter on an iter object returns itself
iter.iter <- function(obj, ...) {
  obj
}

# default method creates an iterator from a vector or list
iter.default <- function(obj, checkFunc=function(...) TRUE, recycle=FALSE, ...) {
  state <- new.env()
  state$i <- 0L
  state$obj <- obj
  n <- length(obj)
  it <- list(state=state, length=n, checkFunc=checkFunc, recycle=recycle)
  class(it) <- c('containeriter', 'iter')
  it
}

# allow a matrix to be iterated over in different ways
iter.matrix <- function(obj, by=c('column', 'cell', 'row'), chunksize=1L,
                        checkFunc=function(...) TRUE, recycle=FALSE, ...) {
  by <- match.arg(by)
  if ((chunksize > 1L) && (by=='cell')) {
    warning("Chunksize greater than 1 not allowed when using by='cell'\n  Setting chunksize=1")
    chunksize <- 1L
  }
  state <- new.env()
  state$i <- 0L
  state$obj <- obj
  n <- switch(by, column=ncol(obj), row=nrow(obj), length(obj))
  it <- list(state=state, by=by, length=n, checkFunc=checkFunc,
             recycle=recycle, chunksize=chunksize)
  class(it) <- c('matrixiter', 'iter')
  it
}

# allow a data frame to be iterated over in different ways
iter.data.frame <- function(obj, by=c('column', 'row'),
                            checkFunc=function(...) TRUE, recycle=FALSE, ...) {
  by <- match.arg(by)
  state <- new.env()
  state$i <- 0L
  state$obj <- obj
  n <- switch(by, column=length(obj), nrow(obj))
  it <- list(state=state, by=by, length=n, checkFunc=checkFunc,
             recycle=recycle)
  class(it) <- c('dataframeiter', 'iter')
  it
}

# allow a closure to be turned into an iterator object
iter.function <- function(obj, checkFunc=function(...) TRUE,
                          recycle=FALSE, ...) {
  state <- new.env()
  state$i <- 0L
  state$fun <- obj
  args <- !is.null(formals(obj))
  it <- list(state=state, args=args, checkFunc=checkFunc, 
             recycle=recycle)
  class(it) <- c('funiter', 'iter')
  it
}

getIterVal <- function(obj, plus, ...) {
  UseMethod('getIterVal')
}

getIterVal.containeriter <- function(obj, plus=0L, ...) {
  i <- obj$state$i + plus
  if (i > obj$length)
    stop('SubscriptOutOfBounds', call.=FALSE)
  obj$state$obj[[i]]
}

getIterVal.matrixiter <- function(obj, plus=0L, ...) {
  i <- obj$state$i + plus
  n <- obj$length
  if (i > n)
    stop('SubscriptOutOfBounds', call.=FALSE)
  j <- i + obj$chunksize - 1L
  switch(obj$by,
         column=obj$state$obj[, i:min(j, n), drop=FALSE],
         row=obj$state$obj[i:min(j, n), , drop=FALSE],
         obj$state$obj[[i]])
}

getIterVal.dataframeiter <- function(obj, plus=0L, check=TRUE, ...) {
  i <- obj$state$i + plus
  n <- obj$length
  if (i > n)
    stop('StopIteration', call.=FALSE)
  switch(obj$by,
         column=obj$state$obj[, i],
         obj$state$obj[i, ])
}

nextElem <- function(obj, ...) {
  UseMethod('nextElem')
}

nextElem.containeriter <- function(obj, ...) {
  repeat {
    tryCatch({
      if (obj$checkFunc(getIterVal(obj,1L))) {
        obj$state$i <- obj$state$i + 1L
        return(getIterVal(obj))
      }
      obj$state$i <- obj$state$i + 1L
    }, error=function(e) {
      if (any(nzchar(e$message))) {
        if (identical(e$message, "SubscriptOutOfBounds")) {
          if (obj$recycle) {
            obj$state$i <- 0L
          }
          else {
            stop('StopIteration', call.=FALSE)
          }
        }
        else {
          stop(e$message, call.=FALSE)
        }
      }
      else {
        stop('Abort', call.=e)
      }
    })
  }
}

nextElem.matrixiter <- function(obj, ...) {
  repeat {
    tryCatch({
      if (obj$checkFunc(getIterVal(obj,1L))) {
        obj$state$i <- obj$state$i + obj$chunksize
        return(getIterVal(obj,plus=(1L-obj$chunksize)))
      }
      obj$state$i <- obj$state$i + obj$chunksize
    }, error=function(e) {
      if (any(nzchar(e$message))) {
        if (identical(e$message, "SubscriptOutOfBounds") ||
            identical(e$message, "attempt to select more than one element")) {
          if (obj$recycle) {
            obj$state$i <- 0L
          }
          else {
            stop('StopIteration', call.=FALSE)
          }
        }
        else {
          stop(e$message, call.=FALSE)
        }
      }
      else {
        stop('Abort', call.=e)
      }
    })
  }
}

nextElem.dataframeiter <- function(obj, ...) {
  repeat {
    tryCatch({
      if (obj$checkFunc(getIterVal(obj,1L))) {
        obj$state$i <- obj$state$i + 1L
        return(getIterVal(obj))
      }
      obj$state$i <- obj$state$i + 1L
    }, error=function(e) {
      if (any(nzchar(e$message))) {
        if (identical(e$message, "StopIteration")) {
          if (obj$recycle) {
            obj$state$i <- 0L
          }
          else {
            stop('StopIteration', call.=FALSE)
          }
        }
        else {
          stop(e$message, call.=FALSE)
        }
      }
      else {
        stop('Abort', call.=e)
      }
    })
  }
}

nextElem.funiter <- function(obj, ...) {
  repeat {
    tryCatch({
      if (obj$args) {
        val <- obj$state$fun(obj$state$i+1L)
      }
      else {
        val <- obj$state$fun()
      }
      if (obj$checkFunc(val)) {
        if (obj$args) obj$state$i <- obj$state$i + 1L
        return(val)
      }
      if (obj$args) obj$state$i <- obj$state$i + 1L
    }, error=function(e) {
      if (any(nzchar(e$message))) {
        if (identical(e$message, "StopIteration")) {
          if (obj$recycle) {
            if (obj$args) obj$state$i <- 0L
          }
          else {
            stop('StopIteration', call.=FALSE)
          }
        }
        else {
          stop(e$message, call.=FALSE)
        }
      }
      else {
        stop('Abort', call.=e)
      }
    })
  }
}

nextElem.abstractiter <- function(obj, ...) {
  obj$nextElem()
}

#print.containeriter <- function(x, ...) {
#  repr <- sprintf('<%s iterator, current value %d\n',
#                  class(x$state$obj)[1], getIterVal(x))
#  cat(repr)
#}

#print.matrixiter <- function(x, ...) {
#  repr <- sprintf('<%s iterator, current value %d\n',
#                  class(x$state$obj)[1], getIterVal(x))
#  cat(repr)
#}

#print.dataframeiter <- function(x, ...) {
#  repr <- sprintf('<%s iterator, current value %d\n',
#                  class(x$state$obj)[1], getIterVal(x, check=FALSE))
#  cat(repr)
#}

#print.funiter <- function(x, ...) {
#  cat('function iterator\n')
#}

#print.abstractiter <- function(x, ...) {
#  cat(x$toString())
#}

library(iterators)

# test that various values of chunksize
test01 <- function() {
  nr <- 13
  nc <- 21
  mat <- matrix(rnorm(nr * nc), nr)

  for (n in 1:(nc+2)) {
    it <- iter(mat, by='col', chunksize=n)
    bcols <- as.list(it)
    for (bcol in bcols) {
      checkTrue(nrow(bcol) == nr)
      checkTrue(ncol(bcol) <= n && ncol(bcol) >= 1)
    }
    actual <- do.call('cbind', bcols)
    checkEquals(mat, actual)
  }

  for (n in 1:(nr+2)) {
    it <- iter(mat, by='row', chunksize=n)
    brows <- as.list(it)
    for (brow in brows) {
      checkTrue(ncol(bcol) == nc)
      checkTrue(nrow(brow) <= n && nrow(brow) >= 1)
    }
    actual <- do.call('rbind', brows)
    checkEquals(mat, actual)
  }
}

library(iterators)

test00 <- function() {}

# test vector iterator creation
test01 <- function() {
  x <- iter(1:10)
}

# test hasNext, nextElem
test02 <- function() {
  x <- iter(1:10)
  checkEquals(nextElem(x), 1)
  for(i in 1:9) nextElem(x)
  checkException(nextElem(x))
}

# check checkFunc
test03 <- function() {
  x <- iter(1:100, checkFunc=function(i) i%%10==0)
  checkEquals(nextElem(x), 10)
  for(i in 1:9) nextElem(x)
  checkException(nextElem(x))
}

# test matrix iterator creation
test04 <- function() {
  x <- matrix(1:10,ncol=2)
}

# test hasNext, nextElem
test05 <- function() {
  x <- matrix(1:10,ncol=2)
  # by cell
  y <- iter(x,by='cell')
  checkEquals(nextElem(y), 1)
  for(i in 1:9) nextElem(y)
  checkException(nextElem(y))

  # by col
  y <- iter(x,by='column')
  checkEquals(nextElem(y), matrix(1:5, ncol=1))
  nextElem(y)
  checkException(nextElem(y))

  # by row
  y <- iter(x,by='row')
  checkEquals(nextElem(y), matrix(c(1,6),nrow=1))
  for(i in 1:4) nextElem(y)
  checkException(nextElem(y))
}
  
# test checkFunc
test06 <- function() {
  # create a larger matrix
  x <- matrix(1:100, ncol=20)

  # by cell
  y <- iter(x, by='cell', checkFunc=function(i) i%%10==0)
  checkEquals(nextElem(y), 10)
  for(i in 1:9) nextElem(y)
  checkException(nextElem(y))

  # by col
  y <- iter(x, by='column', checkFunc=function(i) i[5]%%10==0)
  checkEquals(nextElem(y), as.matrix(x[,2]))
  for(i in 1:9) nextElem(y)
  checkException(nextElem(y))

  # by row
  # create an easier matrix to deal with
  x <- matrix(1:100, nrow=20, byrow=TRUE)
  y <- iter(x, by='row', checkFunc=function(i) i[5]%%10==0)
  checkEquals(as.vector(nextElem(y)), x[2,])
  for(i in 1:9) nextElem(y)
  checkException(nextElem(y))
} 

# test data frame iterator creation
test07 <- function() {
  x <- data.frame(1:10, 11:20)
  y <- iter(x)
}
# test hasNext, nextElem
test08 <- function() {
  x <- data.frame(1:10, 11:20)
  # by row
  y <- iter(x, by='row')
  checkEquals(nextElem(y), x[1,])
  for(i in 1:9) nextElem(y)
  checkException(nextElem(y))

  # by col
  y <- iter(x, by='column')
  checkEquals(nextElem(y), x[,1])
  nextElem(y)
  checkException(nextElem(y))
}

# test checkFunc
test09 <- function() {
  x <- data.frame(1:10, 11:20)
  # by row
  y <- iter(x, by='row', checkFunc=function(i) i[[1]][1]%%2==0)
  checkEquals(nextElem(y),x[2,])
  for(i in 1:4) nextElem(y)
  checkException(nextElem(y))

  # by col
  y <- iter(x, by='column', checkFunc=function(i) i[[1]][1]%%11==0)
  checkEquals(nextElem(y), x[,2])
  checkException(nextElem(y))
}

# test function iterator creation
# we need to test a function that takes no arguement as
# well as one that takes the index
test10 <- function() {
  noArgFunc <- function() 1
  needArgFunc <- function(i)
    if(i>100)
      stop('too high')
    else
      i
}
  
# test hasNext, nextElem
test11 <- function() {
  noArgFunc <- function() 1
  needArgFunc <- function(i) if(i>100)      stop('too high')    else      i
  y <- iter(noArgFunc)
  checkEquals(nextElem(y), 1)
  nextElem(y)

  y <- iter(needArgFunc)
  checkEquals(nextElem(y), 1)
  for (i in 1:99) nextElem(y)
  checkException(nextElem(y))
}
 
# test checkFunc
test12 <- function() {
  noArgFunc <- function() 1
  needArgFunc <- function(i)
    if(i>100)
      stop('too high')
    else
      i
  y <- iter(noArgFunc, checkFunc=function(i) i==1)
  checkEquals(nextElem(y), 1)
  nextElem(y)

  y <- iter(needArgFunc, checkFunc=function(i) i%%10==0)
  checkEquals(nextElem(y), 10)
  for(i in 1:9) nextElem(y)
  checkException(nextElem(y))
}

#!/bin/sh

LOGFILE=test.log

R --vanilla --slave > ${LOGFILE} 2>&1 <<'EOF'
library(iterators)
library(RUnit)

options(warn=1)
options(showWarnCalls=TRUE)

cat('Starting test at', date(), '\n')

tests <- c('basicTest.R', 'iapplyTest.R', 'isplitTest.R',
           'icountnTest.R', 'chunksizeTest.R', 'recycleTest.R')

errcase <- list()
for (f in tests) {
  cat('\nRunning test file:', f, '\n')
  t <- runTestFile(f)
  e <- getErrors(t)
  if (e$nErr + e$nFail > 0) {
    errcase <- c(errcase, t)
    print(t)
  }
}

if (length(errcase) == 0) {
  cat('*** Ran all tests successfully ***\n')
} else {
  cat('!!! Encountered', length(errcase), 'problems !!!\n')
  for (t in errcase) {
    print(t)
  }
}

cat('Finished test at', date(), '\n')
EOF

% \VignetteIndexEntry{iterators Manual}
% \VignetteDepends{iterators}
% \VignettePackage{iterators}
\documentclass[12pt]{article}
\usepackage{amsmath}
\usepackage[pdftex]{graphicx}
\usepackage{color}
\usepackage{xspace}
\usepackage{fancyvrb}
\usepackage{fancyhdr}
    \usepackage[
         colorlinks=true,
         linkcolor=blue,
         citecolor=blue,
         urlcolor=blue]
         {hyperref}
         \usepackage{lscape}

\usepackage{Sweave}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define new colors for use
\definecolor{darkgreen}{rgb}{0,0.6,0}
\definecolor{darkred}{rgb}{0.6,0.0,0}
\definecolor{lightbrown}{rgb}{1,0.9,0.8}
\definecolor{brown}{rgb}{0.6,0.3,0.3}
\definecolor{darkblue}{rgb}{0,0,0.8}
\definecolor{darkmagenta}{rgb}{0.5,0,0.5}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\newcommand{\bld}[1]{\mbox{\boldmath $#1$}}
\newcommand{\shell}[1]{\mbox{$#1$}}
\renewcommand{\vec}[1]{\mbox{\bf {#1}}}

\newcommand{\ReallySmallSpacing}{\renewcommand{\baselinestretch}{.6}\Large\normalsize}
\newcommand{\SmallSpacing}{\renewcommand{\baselinestretch}{1.1}\Large\normalsize}

\newcommand{\halfs}{\frac{1}{2}}

\setlength{\oddsidemargin}{-.25 truein}
\setlength{\evensidemargin}{0truein}
\setlength{\topmargin}{-0.2truein}
\setlength{\textwidth}{7 truein}
\setlength{\textheight}{8.5 truein}
\setlength{\parindent}{0.20truein}
\setlength{\parskip}{0.10truein}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\pagestyle{fancy}
\lhead{}
\chead{Using The {\tt iterators} Package}
\rhead{}
\lfoot{}
\cfoot{}
\rfoot{\thepage}
\renewcommand{\headrulewidth}{1pt}
\renewcommand{\footrulewidth}{1pt}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\title{Using The {\tt iterators} Package}
\author{Rich Calaway}


\begin{document}

\maketitle

\thispagestyle{empty}
\section{Introduction}

An {\em iterator} is a special type of object that generalizes the notion of 
a looping variable. When passed as an argument to a function that knows what 
to do with it, the iterator supplies a sequence of values. The iterator also 
maintains information about its state, in particular its current index. The
\texttt{iterators} package includes a number of functions for creating 
iterators, the simplest of which is \texttt{iter}, which takes
virtually any R object and turns it into an iterator object. The simplest 
function that operates on iterators is the \texttt{nextElem} function, which 
when given an iterator, returns the next value of the iterator. For example, 
here we create an iterator object from the sequence 1 to 10, and then use 
\texttt{nextElem} to iterate through the values:
<<ex1>>=
library(iterators)
i1 <- iter(1:10)
nextElem(i1)
nextElem(i1)
@

You can create iterators from matrices and data frames, using the \texttt{by} argument to specify whether to iterate by row or column:
<<ex2>>=
istate <- iter(state.x77, by='row')
nextElem(istate)
nextElem(istate)
@

Iterators can also be created from functions, in which case the iterator can be an endless source of values:
<<ex3>>=
ifun <- iter(function() sample(0:9, 4, replace=TRUE))
nextElem(ifun)
nextElem(ifun)
@

For practical applications, iterators can be paired with \texttt{foreach} to obtain parallel results quite easily:
\begin{Schunk}
\begin{Sinput}
> library(foreach)
\end{Sinput}
\begin{Soutput}
foreach: simple, scalable parallel programming from Revolution Analytics
Use Revolution R for scalability, fault tolerance and more.
http://www.revolutionanalytics.com
\end{Soutput}
\begin{Sinput}
> x <- matrix(rnorm(1e+06), ncol = 10000)
> itx <- iter(x, by = "row")
> foreach(i = itx, .combine = c) %dopar% mean(i)
\end{Sinput}
\begin{Soutput}
  [1] -0.0069652059  0.0161112989  0.0080068074 -0.0120020610  0.0017168149
  [6]  0.0139835943 -0.0078172106 -0.0024762273 -0.0031558268 -0.0072662893
 [11] -0.0055142639  0.0015717907 -0.0100842965 -0.0123601527  0.0136420084
 [16] -0.0242922105 -0.0126416949 -0.0052951152  0.0216329326 -0.0262476648
 [21]  0.0041937609  0.0121253368 -0.0110165729  0.0044267635  0.0080241894
 [26]  0.0042995539 -0.0102826632  0.0051185628 -0.0013970812 -0.0172380786
 [31]  0.0096079613  0.0046837729 -0.0080726970  0.0083781727 -0.0234620163
 [36] -0.0099883966  0.0026883628  0.0029367320  0.0205825899  0.0035303940
 [41]  0.0204990426 -0.0010804987 -0.0033665481 -0.0127492019 -0.0147443195
 [46]  0.0027046346  0.0016449793  0.0155575490 -0.0003488394 -0.0079238019
 [51]  0.0086390030 -0.0039033309  0.0168593825 -0.0067189572 -0.0009925288
 [56] -0.0162907048 -0.0059171838  0.0093806072  0.0100886929 -0.0111677408
 [61]  0.0021754963 -0.0056770907  0.0081200698 -0.0029828717 -0.0163704350
 [66]  0.0057266267 -0.0017156156  0.0214172738 -0.0141379874 -0.0126593342
 [71]  0.0087124575  0.0040231519  0.0038515673  0.0066066908  0.0023586046
 [76] -0.0044167901 -0.0090543553  0.0010806096  0.0102288061  0.0039881702
 [81] -0.0054549319 -0.0127997275 -0.0031697122 -0.0016100996 -0.0143468118
 [86]  0.0035904125 -0.0059399479  0.0085565480 -0.0159064868  0.0054120554
 [91] -0.0084420572  0.0194448129 -0.0103192553 -0.0062924628  0.0215069258
 [96]  0.0015749065  0.0109657488  0.0152237842 -0.0057181022  0.0035530715
\end{Soutput}
\end{Schunk}

\section{Some Special Iterators}

The notion of an iterator is new to R, but should be familiar to users of
languages such as Python. The \texttt{iterators} package includes a number of
special functions that generate iterators for some common scenarios. For 
example, the
\texttt{irnorm} function creates an iterator for which each value is drawn
from a specified random normal distribution:
<<ex5>>=
library(iterators)
itrn <- irnorm(10)
nextElem(itrn)
nextElem(itrn)
@

Similarly, the \texttt{irunif}, \texttt{irbinom}, and \texttt{irpois} functions
create iterators which drawn their values from uniform, binomial, and Poisson
distributions, respectively.

We can then use these functions just as we used \texttt{irnorm}:
<<ex6>>=
itru <- irunif(10)
nextElem(itru)
nextElem(itru)
@

The \texttt{icount} function returns an iterator that counts starting from one:
<<ex7>>=
it <- icount(3)
nextElem(it)
nextElem(it)
nextElem(it)
@

\end{document}

% \VignetteIndexEntry{Writing Custom Iterators}
% \VignetteDepends{iterators}
% \VignettePackage{iterators}
\documentclass[12pt]{article}
\usepackage{amsmath}
\usepackage[pdftex]{graphicx}
\usepackage{color}
\usepackage{xspace}
\usepackage{fancyvrb}
\usepackage{fancyhdr}
    \usepackage[
         colorlinks=true,
         linkcolor=blue,
         citecolor=blue,
         urlcolor=blue]
         {hyperref}
         \usepackage{lscape}

\usepackage{Sweave}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define new colors for use
\definecolor{darkgreen}{rgb}{0,0.6,0}
\definecolor{darkred}{rgb}{0.6,0.0,0}
\definecolor{lightbrown}{rgb}{1,0.9,0.8}
\definecolor{brown}{rgb}{0.6,0.3,0.3}
\definecolor{darkblue}{rgb}{0,0,0.8}
\definecolor{darkmagenta}{rgb}{0.5,0,0.5}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\newcommand{\bld}[1]{\mbox{\boldmath $#1$}}
\newcommand{\shell}[1]{\mbox{$#1$}}
\renewcommand{\vec}[1]{\mbox{\bf {#1}}}

\newcommand{\ReallySmallSpacing}{\renewcommand{\baselinestretch}{.6}\Large\normalsize}
\newcommand{\SmallSpacing}{\renewcommand{\baselinestretch}{1.1}\Large\normalsize}

\newcommand{\halfs}{\frac{1}{2}}

\setlength{\oddsidemargin}{-.25 truein}
\setlength{\evensidemargin}{0truein}
\setlength{\topmargin}{-0.2truein}
\setlength{\textwidth}{7 truein}
\setlength{\textheight}{8.5 truein}
\setlength{\parindent}{0.20truein}
\setlength{\parskip}{0.10truein}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\pagestyle{fancy}
\lhead{}
\chead{Writing Custom Iterators}
\rhead{}
\lfoot{}
\cfoot{}
\rfoot{\thepage}
\renewcommand{\headrulewidth}{1pt}
\renewcommand{\footrulewidth}{1pt}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\title{Writing Custom Iterators}
\author{Steve Weston}


\begin{document}

\maketitle

\thispagestyle{empty}
\section{Introduction}

<<loadLibs,echo=FALSE,results=hide>>=
library(iterators)
@

An {\em iterator} is a special type of object that supplies data on
demand, one element\footnote{An ``element'' in this case can be basically
any object.  I don't mean to suggest that the data is necessarily returned
as scalar values, for example.} at a time.  This is a nice abstraction
that can help simplify many programs.  Iterators are particularly useful
in parallel computing, since they facilitate splitting a problem into
smaller pieces that can then be executed in parallel.

Iterators can also be used to reduce the total memory that is needed at
any one time.  For example, if you want to process the lines of text in
a file, it is common to write a loop that reads the file one line at a
time, rather than reading the entire file in order to avoid running out
of memory on huge files.  That's the basic idea of iterators.  Iterators
provide a standard method for getting the next element, which allows us
to write functions that take an iterator as an argument to provide a
source of data.  The function doesn't need to know what kind of iterator
it is.  It just needs to know how to get another piece of data.  The
data could be coming from a file, a database, a vector, or it could be
dynamically generated.

There are a number of iterators that come in the \texttt{iterators}
package.  The \texttt{iapply} function allows you to iterate over
arrays, in much the same way as the standard \texttt{apply} function.
\texttt{apply} has fixed rules on how the results are returned, which
may require you to reshape the results, which can be inefficient, as
well as inconvenient.  But since \texttt{iapply} doesn't process any
data or combine the results, it is more flexible.  You can use
\texttt{iapply} with the \texttt{foreach} package to perform a parallel
\texttt{apply} operation, and combine the results any way you want via
the \texttt{.combine} argument to \texttt{foreach}.

Another iterator that comes in the \texttt{iterators} package is the
\texttt{isplit} function, which works much like the standard
\texttt{split} function.
\texttt{split} returns a list containing all of the data divided into
groups.  \texttt{isplit} only generates one group at a time, as they are
needed, which can reduce the amount memory that is needed.

But of course, there will be times when you need an iterator that isn't
provided by the \texttt{iterators} package.  That is when you need to
write your own custom iterator.  Fortunately, that is fairly easy to do.

\section{What methods are needed for an iterator?}

Basically, an iterator is an S3 object whose base class is \texttt{iter}, and
has \texttt{iter} and \texttt{nextElem} methods.  The purpose of the
\texttt{iter} method is to return an iterator for the specified object.
For iterators, that usually just means returning itself, which seems odd
at first.  But the \texttt{iter} method can be defined for other objects
that don't define a \texttt{nextElem} method.  We call those objects
{\em iterables}, meaning that you can iterate over them.  The
\texttt{iterators} package defines \texttt{iter} methods for vectors,
lists, matrices, and data frames, making those objects iterables.  By
defining an \texttt{iter} method for iterators, they can be used in the
same context as an iterable, which can be convenient.  For example, the
\texttt{foreach} function takes iterables as arguments.  It calls the
\texttt{iter} method on those arguments in order to create iterators for
them.  By defining the \texttt{iter} method for all iterators, we can
pass iterators to \texttt{foreach} that we created using any method we
choose.  Thus, we can pass vectors, lists, or iterators to
\texttt{foreach}, and they are all processed by \texttt{foreach} in
exactly the same way.

The \texttt{iterators} package comes with an \texttt{iter} method
defined for the \texttt{iter} class that simply returns itself.  That is
usually all that is needed for an iterator.  However, if you want to
create an iterator for some existing class, you can do that by writing
an \texttt{iter} method that returns an appropriate iterator.  That
will allow you to pass an instance of your class to \texttt{foreach},
which will automatically convert it into an iterator.  The alternative
is to write your own function that takes arbitrary arguments, and
returns an iterator.  You can choose whichever method is most natural.

The most important method required for iterators is \texttt{nextElem}.
This simply returns the next value, or throws an error.  Calling the
\texttt{stop} function with the string \texttt{'StopIteration'} indicates that
there are no more values available in the iterator.

Now before we write our own iterator, let's try calling the
\texttt{iter} and \texttt{nextElem} methods on an existing one.
Since a list is an iterable, we can create an iterator for that list
by calling \texttt{iter} on it:

<<iterable1>>=
it <- iter(list(1:2, 3:4))
@

We can now call \texttt{nextElem} on the resulting iterator to get the
values from the list:

<<iterable2>>=
nextElem(it)
nextElem(it)
tryCatch(nextElem(it), error=function(e) e)
@

As you can see, it is possible to call these methods manually, but it's
somewhat awkward, since you have to handle the \texttt{'StopIteration'} error.
Later on, we'll see one solution to this difficulty, although, in
general, you don't call these method explicitly.

\section{A simple iterator}

It's time to show the implementation of a very simple iterator.  Although I've
made it sound like you have to write your own \texttt{iter} and
\texttt{nextElem} methods, you can inherit them.  In fact, that's what
all of the following examples do.  I do that by inheriting
from the \texttt{abstractiter} class.  The \texttt{abstractiter} class uses 
the standard \texttt{iter} method which returns itself, and defines a 
\texttt{nextElem} method that calls the \texttt{nextElem} element of the object.
Let's take a look at the implementation of these two methods:

<<nextElem.abstractiter>>=
iterators:::iter.iter
iterators:::nextElem.abstractiter
@

Now here's a function that creates a very simple iterator that uses
these two methods:

<<iter1>>=
iforever <- function(x) {
  nextEl <- function() x
  obj <- list(nextElem=nextEl)
  class(obj) <- c('iforever', 'abstractiter', 'iter')
  obj
}
@

Note that I called the internal function \texttt{nextEl} rather than
\texttt{nextElem}.  I do that by convention to avoid masking the standard
\texttt{nextElem} generic function.  That causes problems when you want your
iterator to call the \texttt{nextElem} method of another iterator, which
can be quite useful, as we'll see in a later example.

We create an instance of this iterator by calling the \texttt{iforever}
function, and then use it by calling the \texttt{nextElem} method on the
resulting object:

<<runiter1>>=
it <- iforever(42)
nextElem(it)
nextElem(it)
@

You can also get values from an iterator using \texttt{as.list}.  But
since this is an infinite iterator, you need to use the \texttt{n} argument
to avoid using up a lot of memory and time:

<<runiter1.part2>>=
unlist(as.list(it, n=6))
@

Notice that it doesn't make sense to implement this iterator by defining
a new \texttt{iter} method, since there is no natural iterable on which
to dispatch.  The only argument that we need is the object for the
iterator to return, which can be of any type.  Instead, we implement
this iterator by defining a normal function that returns the iterator.

This iterator is quite simple to implement, and possibly even
useful.\footnote{Be careful how you use this iterator!  If you pass it
to \texttt{foreach}, it will result in an infinite loop unless you pair
it with a non-infinite iterator.  Also, {\em never} pass this to the
\texttt{as.list} function without the \texttt{n} argument.} The iterator
returned by \texttt{iforever} is a list that has a single element named
\texttt{nextElem}, whose value is a function that returns the value of
\texttt{x}.  Because we are subclassing \texttt{abstractiter}, we inherit a
\texttt{nextElem} method that will call this function, and because we
are subclassing \texttt{iter}, we inherit an \texttt{iter} method that will
return itself.

Of course, the reason this iterator is so simple is because it doesn't
contain any state.  Most iterators need to contain some state, or it
will be difficult to make it return different values and eventually
stop.  Managing the state is usually the real trick to writing iterators.

\section{A stateful iterator}

Let's modify the previous iterator to put a limit on the number of values
that it returns.  I'll call the new function \texttt{irep}, and give it
another argument called \texttt{times}:

<<iter2>>=
irep <- function(x, times) {
  nextEl <- function() {
    if (times > 0)
      times <<- times - 1
    else
      stop('StopIteration')

    x
  }

  obj <- list(nextElem=nextEl)
  class(obj) <- c('irep', 'abstractiter', 'iter')
  obj
}
@

Now let's try it out:

<<runiter2>>=
it <- irep(7, 6)
unlist(as.list(it))
@

The real difference between \texttt{iforever} and \texttt{irep} is in the
function that gets called by the \texttt{nextElem} method.  This function not
only accesses the values of the variables \texttt{x} and \texttt{times}, but 
it also modifies the value of \texttt{times}.  This is accomplished by means 
of the ``\verb=<<-='' \footnote{It's commonly believed that ``$<<-$'' is only 
used to set variables in the global environment, but that isn't true.  I think 
of it as an {\em inheriting} assignment operator.} operator, and the magic of 
lexical scoping.  Technically, this kind of function is called a 
{\em closure}, and is a somewhat advanced feature of \texttt{R}.  The 
important thing to remember is that \texttt{nextEl} is able to get the value 
of variables that were passed as arguments to \texttt{irep}, and it can modify 
those values using the ``\verb=<<-='' operator.  These are {\em not} global 
variables: they are defined in the enclosing environment of the 
\texttt{nextEl} function.  You can create as many
iterators as you want using the \texttt{irep} function, and they will all work
as expected without conflicts.

Note that this iterator only uses the arguments to \texttt{irep} to store its
state.  If any other state variables are needed, they can be defined anywhere
inside the \texttt{irep} function.

\section{Using an iterator inside an iterator}

The previous section described a general way of writing custom iterators.
Almost any iterator can be written using those basic techniques.  At times, it
may be simpler to make use of an existing iterator to implement a new iterator.
Let's say that you need an iterator that splits a vector into subvectors.  That
can allow you to process the vector in parallel, but still use vector
operations, which is essential to getting good sequential performance in R.
The following function returns just such an iterator:

<<iter3>>=
ivector <- function(x, ...) {
 i <- 1
 it <- idiv(length(x), ...)

 nextEl <- function() {
   n <- nextElem(it)
   ix <- seq(i, length=n)
   i <<- i + n
   x[ix]
 }

 obj <- list(nextElem=nextEl)
 class(obj) <- c('ivector', 'abstractiter', 'iter')
 obj
}
@

\texttt{ivector} uses \texttt{...} to pass options on to
\texttt{idiv}.  \texttt{idiv} supports the \texttt{chunks} argument
to split its argument into a specified number of pieces, and the
\texttt{chunkSize} argument to split it into pieces of a specified
maximum size.

Let's create an \texttt{ivector} iterator to split a vector into three
pieces using the \texttt{chunks} argument:

<<runiter3>>=
it <- ivector(1:25, chunks=3)
as.list(it)
@

Note that the \texttt{nextEl} function doesn't seem to throw a
\texttt{StopIteration} exception.  It is actually throwing it
indirectly, by calling \texttt{nextElem} on the iterator created via the
\texttt{idiv} function.  This function is fairly simple, because most of
the tricky stuff is handled by \texttt{idiv}.  \texttt{ivector} focuses
on operating on the vector.

It should be clear that only minor modification need to be made to this
function to create an iterator over the blocks of rows or columns of a
matrix or data frame.  But I'll leave that as an exercise for the
reader.

\section{Adding a \texttt{hasNext} method to an iterator}

At times it would be nice to write a loop that explicitly gets the
values of an iterator.  Although that is certainly possible with a
standard iterator, it requires some rather awkward error handling.  One
solution to this problem is to add a method that indicates whether there
is another value available in the iterator.  Then you can write a simple
while loop that stops when there are no more values.

One way to do that would be to define a new S3 method called \texttt{hasNext}.
Here's the definition of a \texttt{hasNext} generic function:

<<generichasnext>>=
hasNext <- function(obj, ...) {
  UseMethod('hasNext')
}
@

We also need to define \texttt{hasNext} method for a iterator class
that we'll call \texttt{ihasNext}:

<<hasnextmethod>>=
hasNext.ihasNext <- function(obj, ...) {
  obj$hasNext()
}
@

As you can see, an \texttt{ihasNext} object must be a list with a 
\texttt{hasNext} element that is a function.  That's the same technique that 
the \texttt{abstractiter} class uses to implement the \texttt{nextElem} method.

Now we'll define a function, called \texttt{ihasNext}, that takes an
arbitrary iterator and returns returns an \texttt{ihasNext} iterator that
wraps the specified iterator.  That allows us to turn any iterator into
an \texttt{ihasNext} iterator, thus providing it with a \texttt{hasNext}
method:\footnote{Thanks to Hadley Wickham for contributing this
function, which I only hacked up a little.  You can also find this
function, along with \texttt{hasNext} and \texttt{hasNext.ihasNext} in
the examples directory of the iterators packages.}

<<ihasnext>>=
ihasNext <- function(it) {
  if (!is.null(it$hasNext)) return(it)
  cache <- NULL
  has_next <- NA

  nextEl <- function() {
    if (!hasNx())
      stop('StopIteration', call.=FALSE)
    has_next <<- NA
    cache
  }

  hasNx <- function() {
    if (!is.na(has_next)) return(has_next)
    tryCatch({
      cache <<- nextElem(it)
      has_next <<- TRUE
    },
    error=function(e) {
      if (identical(conditionMessage(e), 'StopIteration')) {
        has_next <<- FALSE
      } else {
        stop(e)
      }
    })
    has_next
  }

  obj <- list(nextElem=nextEl, hasNext=hasNx)
  class(obj) <- c('ihasNext', 'abstractiter', 'iter')
  obj
}
@

When the \texttt{hasNext} method is called, it calls the
\texttt{nextElem} method on the underlying iterator, and the resulting
value is saved.  That value is then passed to the user when
\texttt{nextElem} is called.  Of course, it also does the right thing if
you don't call \texttt{hasNext}, or if you call it multiple times before
calling \texttt{nextElem}.

So now we can easily create an \texttt{icount} iterator, and get its values
in a while loop, without having to do any messy error handling:

<<hasnextexample>>=
it <- ihasNext(icount(3))
while (hasNext(it)) {
  print(nextElem(it))
}
@

\section{A recycling iterator}

The \texttt{ihasNext} function from the previous section is an
interesting example of a function that takes an iterator and returns an
iterator that wraps the specified iterator.  In that case, we wanted to
add another method to the iterator.  In this example, we'll return an
iterator that recycles the values of the wrapped iterator:\footnote{
Actually, some of the standard \texttt{iter} methods support a
\texttt{recycle} argument.  But this is a nice example, and a more
general solution, since it works on any iterator.}

<<recyle>>=
irecycle <- function(it) {
  values <- as.list(iter(it))
  i <- length(values)

  nextEl <- function() {
    i <<- i + 1
    if (i > length(values)) i <<- 1
    values[[i]]
  }

  obj <- list(nextElem=nextEl)
  class(obj) <- c('irecycle', 'abstractiter', 'iter')
  obj
}
@

This is fairly nice, but note that this is another one of those
infinite iterators that we need to be careful about.  Also, make sure
that you don't pass an infinite iterator to \texttt{irecycle}.  That
would be pointless of course, since there's no reason to recycle an
iterator that never ends.  It would be possible to write this to avoid
that problem by not grabbing all of the values right up front, but you
would still end up saving values that will never be recycled, so I've
opted to keep this simple.

Let's try it out:

<<recyleexample>>=
it <- irecycle(icount(3))
unlist(as.list(it, n=9))
@

\section{Limiting infinite iterators}

I was tempted to add an argument to the \texttt{irecycle} function to
limit the number of values that it returns, because sometimes you want
to recycle for awhile, but not forever.  I didn't do that, because
rather than make \texttt{irecycle} more complicated, I decided to write
yet another function that takes an iterator and returns a modified
iterator to handle that task:

<<ilimit>>=
ilimit <- function(it, times) {
  it <- iter(it)

  nextEl <- function() {
    if (times > 0)
      times <<- times - 1
    else
      stop('StopIteration')

    nextElem(it)
  }

  obj <- list(nextElem=nextEl)
  class(obj) <- c('ilimit', 'abstractiter', 'iter')
  obj
}
@

Note that this looks an awful lot like the \texttt{irep} function that
we implemented previously.  In fact, using \texttt{ilimit}, we can
implement \texttt{irep} using \texttt{iforever} much more simply, and
without duplication of code:

<<irep2>>=
irep2 <- function(x, times)
  ilimit(iforever(x), times)
@

To demonstrate \texttt{irep2}, I'll use \texttt{ihasNext} and a while
loop:

<<testirep2>>=
it <- ihasNext(irep2('foo', 3))
while (hasNext(it)) {
  print(nextElem(it))
}
@

Here's one last example.  Let's recycle a vector three times using
\texttt{ilimit}, and convert it back into a vector using
\texttt{as.list} and \texttt{unlist}:

<<testirecycle>>=
iterable <- 1:3
n <- 3
it <- ilimit(irecycle(iterable), n * length(iterable))
unlist(as.list(it))
@

Sort of a complicated version of:
<<rep>>=
rep(iterable, n)
@
Aren't iterators fun?

\section{Conclusion}

Writing your own iterators can be quite simple, and yet is very useful
and powerful.  It provides a very effective way to extend the
capabilities of other packages that use iterators, such as the
\texttt{foreach} package.  By writing iterators that wrap other
iterators, it is possible to put together a powerful and flexible set of
tools that work well together, and that can solve many of the complex
problems that come up in parallel computing.

\end{document}

\name{isplit}
\alias{isplit}
\title{Split Iterator}
\description{
  Returns an iterator that divides the data in the vector \code{x}
  into the groups defined by \code{f}.
}
\usage{
isplit(x, f, drop=FALSE, \dots)
}
\arguments{
  \item{x}{vector or data frame of values to be split into groups.}
  \item{f}{a factor or list of factors used to categorize \code{x}.}
  \item{drop}{logical indicating if levels that do not occur should be dropped.}
  \item{\dots}{current ignored.}
}
\value{
  The split iterator.
}
\seealso{
  \code{\link{split}}
}
\examples{
x <- rnorm(200)
f <- factor(sample(1:10, length(x), replace=TRUE))

it <- isplit(x, f)
expected <- split(x, f)

for (i in expected) {
  actual <- nextElem(it)
  stopifnot(actual$value == i)
}
}
\keyword{utilities}

\name{iterators-package}
\alias{iterators-package}
\alias{iterators}
\docType{package}
\title{
The Iterators Package
}
\description{
The iterators package provides tools for iterating over various R data structures.
Iterators are available for vectors, lists, matrices, data frames, and files.
By following very simple conventions, new iterators can be written to support
any type of data source, such as database queries or dynamically generated
data.
}
\details{
Further information is available in the following help topics:
\tabular{ll}{
\code{iter} \tab Generic function used to create iterator objects.\cr
\code{nextElem} \tab Generic function used to get the next element of a iterator.\cr
\code{icount} \tab A function used to create a counting iterator.\cr
\code{idiv} \tab A function used to create a number dividing iterator.\cr
\code{ireadLines} \tab A function used to create a file reading iterator.\cr
}

For a complete list of functions with individual help pages,
use \code{library(help="iterators")}.
}
\keyword{package}

\name{irnorm}
\alias{irnorm}
\alias{irunif}
\alias{irbinom}
\alias{irnbinom}
\alias{irpois}
\title{Random Number Iterators}
\description{
  These function returns an iterators that return random numbers
  of various distributions.
  Each one is a wrapper around a standard \code{R} function.
}
\usage{
irnorm(..., count)
irunif(..., count)
irbinom(..., count)
irnbinom(..., count)
irpois(..., count)
}
\arguments{
  \item{count}{number of times that the iterator will fire.
               If not specified, it will fire values forever.}
  \item{\dots}{arguments to pass to the underlying \code{rnorm} function.}
}
\value{
  An iterator that is a wrapper around the corresponding random number
  generator function.
}
\examples{
  # create an iterator that returns three random numbers
  it <- irnorm(1, count=3)
  nextElem(it)
  nextElem(it)
  nextElem(it)
  try(nextElem(it))  # expect a StopIteration exception
}
\keyword{utilities}

\name{iter}
\alias{iter}
\alias{iter.default}
\alias{iter.iter}
\alias{iter.matrix}
\alias{iter.data.frame}
\alias{iter.function}
\title{Iterator Factory Functions}
\description{
  \code{iter} is a generic function used to create iterator objects.
}
\usage{
iter(obj, \dots)

\method{iter}{default}(obj, checkFunc=function(...) TRUE, recycle=FALSE,
\dots)
\method{iter}{iter}(obj, \dots)
\method{iter}{matrix}(obj, by=c('column', 'cell', 'row'), chunksize=1L,
checkFunc=function(...) TRUE, recycle=FALSE, \dots)
\method{iter}{data.frame}(obj, by=c('column', 'row'),
checkFunc=function(...) TRUE, recycle=FALSE, \dots)
\method{iter}{function}(obj, checkFunc=function(...) TRUE,
recycle=FALSE, \dots)
}
\arguments{
  \item{obj}{an object from which to generate an iterator.}
  \item{by}{how to iterate.}
  \item{chunksize}{the number of elements of \code{by} to return with
    each call to \code{nextElem}.}
  \item{checkFunc}{a function which, when passed an iterator value,
    return \code{TRUE} or \code{FALSE}.  If \code{FALSE}, the value is
    skipped in the iteration.}
  \item{recycle}{a boolean describing whether the iterator should reset
    after running through all it's values.}
  \item{\dots}{additional arguments affecting the iterator.}
}
\value{
  The iterator.
}
\examples{
  # a vector iterator
  i1 <- iter(1:3)
  nextElem(i1)
  nextElem(i1)
  nextElem(i1)

  # a vector iterator with a checkFunc
  i1 <- iter(1:3, checkFunc=function(i) i \%\% 2 == 0)
  nextElem(i1)

  # a data frame iterator by column
  i2 <- iter(data.frame(x=1:3, y=10, z=c('a', 'b', 'c')))
  nextElem(i2)
  nextElem(i2)
  nextElem(i2)

  # a data frame iterator by row
  i3 <- iter(data.frame(x=1:3, y=10), by='row')
  nextElem(i3)
  nextElem(i3)
  nextElem(i3)

  # a function iterator
  i4 <- iter(function() rnorm(1))
  nextElem(i4)
  nextElem(i4)
  nextElem(i4)
}
\keyword{methods}

\name{iapply}
\alias{iapply}
\title{Array/Apply Iterator}
\description{
  Returns an iterator over an array, which iterates over the 
  array in much the same manner
  as the \code{apply} function.
}
\usage{
iapply(X, MARGIN)
}
\arguments{
  \item{X}{the array to iterate over.}
  \item{MARGIN}{a vector of subscripts.
    \code{1} indicates the first dimension (rows), \code{2}
    indicates the second dimension (columns), etc.}
}
\value{
  The apply iterator.
}
\seealso{
  \code{\link{apply}}
}
\examples{
a <- array(1:8, c(2, 2, 2))

# iterate over all the matrices
it <- iapply(a, 3)
as.list(it)

# iterate over all the columns of all the matrices
it <- iapply(a, c(2, 3))
as.list(it)

# iterate over all the rows of all the matrices
it <- iapply(a, c(1, 3))
as.list(it)
}
\keyword{utilities}

\name{nextElem}
\alias{nextElem}
\alias{nextElem.containeriter}
\alias{nextElem.funiter}
\title{Get Next Element of Iterator}
\description{
  \code{nextElem} is a generic function used to produce values. If a
  \code{checkFunc} was specified to the constructor, the potential
  iterated values will be passed to the \code{checkFunc} until the
  \code{checkFunc} returns \code{TRUE}. When the iterator has no more
  values, it calls stop with the message 'StopIteration'.
}
\usage{
nextElem(obj, \dots)

\method{nextElem}{containeriter}(obj, \dots)
\method{nextElem}{funiter}(obj, \dots)
}
\arguments{
  \item{obj}{an iterator object.}
  \item{\dots}{additional arguments that are ignored.}
}
\value{
  The value.
}
\examples{
  it <- iter(c('a', 'b', 'c'))
  print(nextElem(it))
  print(nextElem(it))
  print(nextElem(it))
}
\keyword{methods}

\name{ireadLines}
\alias{ireadLines}
\title{Iterator over Lines of Text from a Connection}
\description{
  Returns an iterator over the lines of text from a connection.
  It is a wrapper around the standard \code{readLines} function.
}
\usage{
ireadLines(con, n=1, ...)
}
\arguments{
  \item{con}{a connection object or a character string.}
  \item{n}{integer.  The maximum number of lines to read.
           Negative values indicate that one should read up to
           the end of the connection.  The default value is 1.}
  \item{\dots}{passed on to the \code{readLines} function.}
}
\value{
  The line reading iterator.
}
\seealso{
  \code{\link[base]{readLines}}
}
\examples{
  # create an iterator over the lines of COPYING
  it <- ireadLines(file.path(R.home(), 'COPYING'))
  nextElem(it)
  nextElem(it)
  nextElem(it)
}
\keyword{utilities}

\name{idiv}
\alias{idiv}
\title{Dividing Iterator}
\description{
  Returns an iterator that returns pieces of numeric value.
}
\usage{
idiv(n, ..., chunks, chunkSize)
}
\arguments{
  \item{n}{number of times that the iterator will fire.
           If not specified, it will count forever.}
  \item{\dots}{unused.}
  \item{chunks}{the number of pieces that \code{n} should be divided into.
           This is useful when you know the number of pieces that you want.
           If specified, then \code{chunkSize} should not be.}
  \item{chunkSize}{the maximum size of the pieces that \code{n}
           should be divided into.
           This is useful when you know the size of the pieces that you want.
           If specified, then \code{chunks} should not be.}
}
\value{
  The dividing iterator.
}
\examples{
  # divide the value 10 into 3 pieces
  it <- idiv(10, chunks=3)
  nextElem(it)
  nextElem(it)
  nextElem(it)
  try(nextElem(it))  # expect a StopIteration exception

  # divide the value 10 into pieces no larger than 3
  it <- idiv(10, chunkSize=3)
  nextElem(it)
  nextElem(it)
  nextElem(it)
  nextElem(it)
  try(nextElem(it))  # expect a StopIteration exception
}
\keyword{utilities}

\name{iread.table}
\alias{iread.table}
\title{Iterator over Rows of a Data Frame Stored in a File}
\description{
  Returns an iterator over the rows of a data frame stored in
  a file in table format.
  It is a wrapper around the standard \code{read.table} function.
}
\usage{
iread.table(file, ..., verbose=FALSE)
}
\arguments{
  \item{file}{the name of the file to read the data from.}
  \item{\dots}{all additional arguments are passed on to the
           \code{read.table} function.  See the documentation for
           \code{read.table} for more information.}
  \item{verbose}{logical value indicating whether or not to print the
           calls to \code{read.table}.}
}
\value{
  The file reading iterator.
}
\note{
In this version of \code{iread.table}, both the \code{read.table}
arguments \code{header} and \code{row.names} must be specified.  This is
because the default values of these arguments depend on the contents of
the beginning of the file.  In order to make the subsequent calls to
\code{read.table} work consistently, the user must specify those
arguments explicitly.  A future version of \code{iread.table} may remove
this requirement.
}
\seealso{
  \code{\link[utils]{read.table}}
}
\keyword{utilities}

\name{makeIwrapper}
\alias{makeIwrapper}
\alias{isample}

\title{Iterator Maker Generator}
\description{
 The \code{makeIwrapper} function makes iterator makers.  The resulting iterator makers all take
 an optional \code{count} argument which specifies the number of times the
 resulting iterator should fire.  The iterators are wrappers around functions
 that return different values each time they are called. The \code{isample} function
 is an example of one such iterator maker (as are \code{irnorm}, \code{irunif}, etc.).
}
\usage{
makeIwrapper(FUN)
isample(..., count)

}
\arguments{
  \item{FUN}{a character string naming a function that generates 
    different values each time it is
    called; typically one of the
    standard random number generator functions.}
  \item{count}{number of times that the iterator will fire.
               If not specified, it will fire values forever.}
  \item{\dots}{arguments to pass to the underlying \code{FUN} function.}
}
\value{
  An iterator that is a wrapper around the corresponding function.
}
\examples{
  # create an iterator maker for the sample function
  mysample <- makeIwrapper('sample')
  # use this iterator maker to generate an iterator
  # that will generate three five member samples from the
  # sequence 1:100
  it <- mysample(1:100, 5, count=3)
  nextElem(it)
  nextElem(it)
  nextElem(it)
  try(nextElem(it))  # expect a StopIteration exception
}
\keyword{utilities}

\name{icount}
\alias{icount}
\alias{icountn}
\title{Counting Iterators}
\description{
  Returns an iterator that counts starting from one.
}
\usage{
icount(count)
icountn(vn)
}
\arguments{
  \item{count}{number of times that the iterator will fire.
  If not specified, it will count forever.}
  \item{vn}{vector of counts.}
}
\value{
  The counting iterator.
}
\examples{
  # create an iterator that counts from 1 to 3.
  it <- icount(3)
  nextElem(it)
  nextElem(it)
  nextElem(it)
  try(nextElem(it))  # expect a StopIteration exception
}
\keyword{utilities}
